#
#  log.rb
#
#  Created by 阳君 on 17/12/12.
#  Copyright © 2017年 DiDiChuXing. All rights reserved.
#

require 'date'
require 'json'

module MapTool
    
    # Usage
    class Log < Command
        
        self.command = 'log'
        self.summary = 'log 日志解析，并保留需要的记录'
        self.description = <<-DESC
        log 日志解析，并保留需要的记录\n
        1. 以文件夹的方式解析，并将解析后的日志存放在文件夹内。\n
        2. 解析时使用字符串匹配，匹配成功的记录会保留 MapTool.log 中。
        DESC
        
        def self.options
            [['--dir', '文件夹地址'],
            ['--match', '匹配的字符串']] + super
        end
    
        attr_accessor :dir   # 文件夹地址
        attr_accessor :match # 匹配的字符串
        
        # 初始化
        def initialize(argv)
            super
            self.dir = argv.option('dir')
            self.match = argv.option('match')
        end
        
        # businrss
        def validate!
            super
            unless self.dir && Dir.exist?(dir)  {
                puts "dir:#{dir} 文件夹不存在".red
                self.banner!
            }
            unless self.match && self.match.length > 0 {
                puts "match 为空".red
                self.banner!
            }
        end
        
        def run
            Dir.chdir(dir) {
                log = ""
                File.delete(LOG_FILE) if File.exist?(LOG_FILE)
                for path in Dir["**/*.log"] do
                    puts "解析 #{path}"
                    File.open(path, "r") { |file|
                        while line = file.gets
                            if line.include?(match)
                                log << line
                            end
                        end
                    }
                end
                File.write(LOG_FILE, log)
                `open #{LOG_FILE}`
            }
        end
        
    end

end

