#
#  tag.rb
#
#  Created by 阳君 on 17/11/14.
#  Copyright © 2017年 DiDiChuXing. All rights reserved.
#

require 'date'
require 'json'

module MapTool
    
    # Usage
    class Tag < Command
        
        self.command = 'tag'
        self.summary = '生成 tag 并推送到 GitLab'
        self.description = <<-DESC
        生成 tag 并推送到 GitLab。\n
        生成的 tag 样式: 库名-version-stage-时间戳-suffix\n
        使用方式：首次或更新版本提交“maptool tag --version=? --stage=? --suffix=?”；只更新时间戳提交“maptool tag”。\n
        注意事项: version标准写法: x.xx.xxx。重大需求和重构升级第一位，第二位和第三位设为00.000；pm 一般需求升级第二位，第三位设为000；bug 修复升级第三位。
        DESC
        
        def self.options
            [['--version', '版本号：x.xx.xxx，默认 1.00.000'],
            ['--stage', '用于区分是否正式: 开发阶段/提测后 (dev/release)，默认 release'],
            ['--suffix', '可扩展的后缀']] + super
        end
    
        TAG_FILE = ".tagfile"
    
        attr_accessor :name    # 名称
        attr_accessor :version # 版本号
        attr_accessor :stage   # 阶段
        attr_accessor :suffix  # 后缀
        
        # 初始化
        def initialize(argv)
            super
            self.version = argv.option('version')
            self.stage = argv.option('stage')
            self.suffix = argv.option('suffix')
        end
        
        # businrss
        def validate!
            super
            unless File.exist?('.git')
                puts "需在项目根目录执行".red
                self.banner!
            end
            if self.version && self.version.size < 8 #x.xx.xxx
                puts "版本号 #{self.version} 不符合规范 x.xx.xxx".red
                self.banner!
            end
        end
    
        def run
            if File.exist?(TAG_FILE) # 文件存在
                save = self.version || self.stage || self.suffix
                self.read_file
                self.commit_file if save
            else
                self.name = File.basename(Dir.pwd)
                self.version = "1.00.000" unless self.version
                self.stage = "release" unless self.stage
                self.commit_file
            end
            self.commit_tag
        end
        
        def read_file
            hash = JSON.parse(File.read(TAG_FILE))
            self.name = hash["name"]
            self.version = hash["version"] unless self.version
            self.stage = hash["stage"] unless self.stage
            self.suffix = hash["suffix"] unless self.suffix
        end
            
        def commit_file
            puts "update #{TAG_FILE}".green
            localChanges = !(`git stash` =~ /No local changes to save/)
            self.write_file
            `git add .`
            `git commit -m "update #{TAG_FILE}"`
            `git push`
            `git stash pop` if localChanges
        end
        
        def write_file
            params = {}
            params["name"] = self.name
            params["version"] = self.version
            params["stage"] = self.stage
            params["suffix"] = self.suffix
            File.write(TAG_FILE, JSON.pretty_generate(params))
        end
        
        def commit_tag
            tag = "#{self.name}-#{self.version}-"
            tag << "#{self.stage}-" if self.stage && self.stage.size > 0
            tag << Time.now.strftime("%Y%m%d%H%M")
            tag << "-#{self.suffix}" if self.suffix && self.suffix.size > 0
            puts "build tag #{tag}".green
            system("yjcocoa git tag --add=#{tag}")
        end
    end

end
